/*
 * HolidayInfo : 祝日情報抽象基本クラス
 */

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public abstract class HolidayInfo
{
	//
	// 内部型定義
	//

	//
	// DateOfYear : 指定年の祝日日付
	// 注記 : 不変オブジェクトとする
	//
	public final class DateOfYear implements Comparable
	{
		//
		// 定数
		//

		private static final String LABEL_HOLIDAY_IN_LIEU	= "(振替休日)";

		//
		// 属性
		//

		private final Calendar dateOfYear;

		//
		// 構築子
		//
		private DateOfYear(
			int year)	//　年(入力)
		{
			if (year < HolidayInfo.this.firstYear
			|| HolidayInfo.this.lastYear < year) {
				throw new IllegalArgumentException();
			}

			dateOfYear = Calendar.getInstance();
			dateOfYear.clear();
			dateOfYear.set(year, month, getDay(year));
		}

		//
		// 状態
		//

		public String name()
		{
			return HolidayInfo.this.name();
		}

		public Calendar calendar()
		{
			Calendar calendar = Calendar.getInstance();
			calendar.clear();
			calendar.setTime(dateOfYear.getTime());
			return calendar;
		}

		public Date date()
		{
			return dateOfYear.getTime();
		}

		public int days()
		{
			return HolidayInfo.this.days;
		}

		// 追加の祝日を返す
		public final List getAdditionalDays(
			Set afterHolidays)	// 以降の祝日
		{
			if (afterHolidays == null) {
				throw new NullPointerException();
			}

			List additionalDays = new ArrayList();

			if (HolidayInfo.this.country.equals("JP")) {
				Calendar date = calendar();
				int year = dateOfYear.get(Calendar.YEAR);

				// 「国民の休日」処理
				if (1986 <= year) {
					// 次の祝日が二日後で振替休日でないとき次の日を国民の休日とする
					Iterator it = afterHolidays.iterator();
					if (it.hasNext()) {
						DateOfYear nextHoliday = (DateOfYear)it.next();
						if ((nextHoliday.calendar().get(Calendar.DAY_OF_YEAR) - date.get(Calendar.DAY_OF_YEAR)) == 2
						&& nextHoliday.name().equals(LABEL_HOLIDAY_IN_LIEU) == false
						&& nextHoliday.calendar().get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY) {
							additionalDays.add(makeNextDateOfYear("国民の休日"));
						}
					}
				}

				// 振替休日処理
				if (1973 < year && year < 2007) {
					// 該当の日の次の平日を振替休日とする
					// ただし次の日を国民の休日としたときは処理しない
					if (additionalDays.isEmpty()) {
						if (date.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
							additionalDays.add(makeNextDateOfYear(LABEL_HOLIDAY_IN_LIEU));
						}
					}
				}
				else if (2007 <= year) {
					// 該当の日以降の平日を振替休日とする
					// ただし次の日を国民の休日としたときは処理しない
					if (additionalDays.isEmpty()) {
						boolean hasAdditional = false;
						boolean inCheck = true;
						while (inCheck) {
							if (date.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
								hasAdditional = true;
								date.add(Calendar.DATE, 1);
								
								// Zaurus SL-C860でカウントアップされない様子があったので明示的に参照してみる
								if (date.get(Calendar.DATE) == calendar().get(Calendar.DATE)) {
								}
							}
							else {
								boolean countUp = false;
								Iterator it = afterHolidays.iterator();
								while (it.hasNext()) {
									if (date.equals(((DateOfYear)it.next()).calendar())) {
										hasAdditional = true;
										date.add(Calendar.DATE, 1);
								
										// Zaurus SL-C860でカウントアップされない様子があったので明示的に参照してみる
										if (date.get(Calendar.DATE) == calendar().get(Calendar.DATE)) {
										}

										countUp = true;
										break;
									}
								}
								inCheck = (countUp != false);
							}
						}

						if (hasAdditional) {
							additionalDays.add(makeDateOfYear(LABEL_HOLIDAY_IN_LIEU, date));
						}
					}
				}
			}
			return additionalDays;
		}

		//
		// 実装
		//

		// 
		private DateOfYear makeNextDateOfYear(
			String strName)	// 名前
		{
			int year = dateOfYear.get(Calendar.YEAR);
			return new DateHolidayInfo(HolidayInfo.this.country,
											strName,
											dateOfYear.get(Calendar.MONTH),
											dateOfYear.get(Calendar.DATE) + 1,
											1,
											year,
											year).getDateOfYear(year);
		}

		// 
		private DateOfYear makeDateOfYear(
			String strName,	// 名前
			Calendar date)	// 作成する日付
		{
			int year = date.get(Calendar.YEAR);
			return new DateHolidayInfo(HolidayInfo.this.country,
											strName,
											date.get(Calendar.MONTH),
											date.get(Calendar.DATE),
											1,
											year,
											year).getDateOfYear(year);
		}

		//
		// Compareableインターフェース実装
		//

		// オブジェクトを比較する
		public int compareTo(
			Object compare)	// 比較対象オブジェクト(入力)
		{
			if (compare == null) {
				throw new NullPointerException();
			}
			else if ((compare instanceof DateOfYear) == false) {
				throw new ClassCastException();
			}
			DateOfYear compareHoliday = (DateOfYear)compare;
			int dateCompareTo = date().compareTo(compareHoliday.date());
			if (dateCompareTo != 0) {
				return dateCompareTo;
			}

			return days() - compareHoliday.days();
		}
	}

	//
	// 属性
	//
	private final String country;
	private final String name;
	private final int month;
	private final int days;
	private final int firstYear;
	private final int lastYear;

	//
	// 構築子
	//
	protected HolidayInfo(
		String strCountry,	// 国(入力)
		String strName,		// 祝日名(入力)
		int nMonth,		// 月(入力)
		int nDays,			// 日数(入力)
		int nFirstYear,	// 開始年(入力)
		int nLastYear)		// 終了年(入力)
	{
		if (strCountry.length() == 0) {
			throw new IllegalArgumentException();
		}
		else if (strName.length() == 0) {
			throw new IllegalArgumentException();
		}
		else if (nMonth < 0 || 12 < nMonth) {
			throw new IllegalArgumentException();
		}
		else if (nLastYear < nFirstYear) {
			throw new IllegalArgumentException();
		}

		country = strCountry;
		name = strName;
		month = nMonth;
		days = nDays;
		firstYear = nFirstYear;
		lastYear = nLastYear;
	}

	//
	// 状態
	//

	// 国を返す
	public final String country()
	{
		return country;
	}

	// 名前を返す
	public final String name()
	{
		return name;
	}

	// 月を返す
	public final int month()
	{
		return month;
	}

	// 指定年の日付を取得する
	public final DateOfYear getDateOfYear(
		int year)	// 祝日取得年(入力)
	{
		DateOfYear holiday = null;
		if (firstYear <= year && year <= lastYear) {
			holiday = new DateOfYear(year);
		}
		return holiday;
	}

	//
	// 実装
	//

	// 指定年の日を返す
	protected abstract int getDay(int year);
}
