/*
 * ICalnedarHolidayListSerializer: iCaalendar(RFC2445)フォーマット出力
 */

import java.io.IOException;
import java.io.Writer;
import java.util.Set;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import environment.EnvInfo;

final public class ICalendarHolidayListSerializer	extends HolidayListSerializer
{
	//
	// 内部型定義
	//

	// FileType: ファイル種別
	// 『Effective Java』「項目21 enum構文をクラスで置き換える」に従って実装
	static public final class FileType
	{
		//
		// クラス属性
		//

		public static final FileType UNKNOWN = new FileType("unknown");
		public static final FileType VCALENDAR = new FileType("vCalendar");
		public static final FileType ICALENDAR = new FileType("iCalendar");

		//
		// 属性
		//
		private final String name;

		//
		// コンストラクタ
		//
		private FileType(String strName) // 名前
		{
			name = strName;
		}

		//
		// 再定義メソッド(Object)
		//

		// 文字列化した結果を返す
		public String toString() {
			return name;
		}
	}

	//
	// 内部型定義
	//
	
	//
	// Factory:
	// Abstract FactoryパターンのConcreteFactoryクラス
	//
	final static class Factory extends HolidayListSerializer.Factory
	{
		//
		// コンストラクタ
		//
		Factory()
		{
			super();
		}
		
		//
		// 再定義必須メソッド
		//
		
		// 作成可能かを返す 
		boolean canCreate(
			String strFileName)	// ファイル名
		{
			if (strFileName == null) {
				throw new NullPointerException();
			}
			else if (strFileName.length() == 0) {
				throw new IllegalArgumentException();
			}

			return (getFileType(strFileName) != FileType.UNKNOWN);
		}
		
		// インスタンスを作成する
		HolidayListSerializer create(
			String strFileName,	// ファイル名
			Set setHoliday)		// 祝日定義の集合
		{
			if (strFileName == null) {
				throw new NullPointerException();
			}
			else if (strFileName.length() == 0) {
				throw new IllegalArgumentException();
			}
			if (setHoliday == null) {
				throw new NullPointerException();
			}
		
			return new ICalendarHolidayListSerializer(getFileType(strFileName), setHoliday);
		}
	}

	//
	// 定数
	//

	static public final String FILE_EXT_ICS = ".ics";
	static public final String FILE_EXT_VCS = ".vcs";

	static private final String OBJECT_ICALENDAR = "VCALENDAR";

	static private final DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");

	//
	// クラスメソッド(実装)
	//

	// ファイル種別を返す
	static private FileType getFileType(
		String strFilePath) // ファイルパス
	{
		if (strFilePath == null) {
			throw new NullPointerException();
		}
		else if (strFilePath.length() == 0) {
			throw new IllegalArgumentException();
		}

		FileType fileType = FileType.UNKNOWN;
		int nExtPos = strFilePath.lastIndexOf('.');
		if (0 < nExtPos) {
			String strExt = strFilePath.substring(nExtPos).toLowerCase();
			if (strExt.compareToIgnoreCase(FILE_EXT_ICS) == 0) {
				fileType = FileType.ICALENDAR;
			}
			else if (strExt.compareToIgnoreCase(FILE_EXT_VCS) == 0) {
				fileType = FileType.VCALENDAR;
			}
		}

		return fileType;
	}

	//
	// 属性
	//

	private final FileType type;
	private final String newLine;

	//
	// コンストラクタ
	//
	private ICalendarHolidayListSerializer(
		FileType fileType, // ファイル種別
		Set setHoliday) // 祝日定義の集合
	{
		super(setHoliday, (fileType == FileType.ICALENDAR ? "UTF-8" : EnvInfo.getInstance().getDefaultEncoding()));

		if (fileType == FileType.UNKNOWN) {
			throw new IllegalArgumentException();
		}

		type = fileType;
		newLine = (fileType == FileType.ICALENDAR ? "\r\n" : EnvInfo.NEWLINE);
	}

	//
	// 実装
	//

	// 行を書きこむ
	private void writeLine(
		String strProperty, // プロパティ名
		String strValue,	// 値
		Writer writer)		// 出力先
		throws IOException
	{
		writer.write(strProperty);
		writer.write(":");
		writer.write(strValue);
		writer.write(newLine);
	}

	// "BEGIN"オブジェクトを書きこむ
	private void writeBeginObject(
		String strObjectName,	// オブジェクト名
		Writer writer) 			// 出力先
		throws IOException
	{
		writeLine("BEGIN", strObjectName, writer);
	}

	// "END"オブジェクトを書きこむ
	private void writeEndObject(
		String strObjectName,	// オブジェクト名
		Writer writer)		 	// 出力先
		throws IOException
	{
		writeLine("END", strObjectName, writer);
	}

	//
	// 再定義メソッド(実装)
	//

	// ヘッダを書きこむ
	protected void writeHeader(
		Writer writer) // 出力先
		throws IOException
	{
		writeBeginObject(OBJECT_ICALENDAR, writer);
		writeLine("PRODID", "tag:contact.d.b.c@gmail.com,2007:StompBox/Holiday", writer);
		writeLine("VERSION", (type == FileType.ICALENDAR ? "2.0" : "1.0"), writer);
	}

	// レコードを書きこむ
	protected void writeRecord(
		HolidayInfo.DateOfYear holidayOfYear, // 
		Writer writer) // 出力先
		throws IOException
	{
		final String OBJECT_EVENT = "VEVENT";
		writeBeginObject(OBJECT_EVENT, writer);

		writeLine("DTSTART;VALUE=DATE", dateFormat.format(holidayOfYear.date()), writer);
		int nDays = holidayOfYear.days();
		writeLine("DURATION", "P" + Integer.toString(nDays) + "D", writer);
		writeLine("TRANSP", "TRANSPARENT", writer);
		// DTSTAMP……はとりあえず省略
		writeLine("SUMMARY", holidayOfYear.name(), writer);

		writeEndObject(OBJECT_EVENT, writer);
	}

	// フッタを書きこむ
	protected void writeFooter(
		Writer writer) // 出力先
		throws IOException
	{
		writeEndObject(OBJECT_ICALENDAR, writer);
	}

}
