/*
 * MainFrame: メインウィンドウ
 */

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Frame;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.TreeSet;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import environment.EnvInfo;
import model.Serializer;

import view.ImplementOperation;
import view.LabelManager;

public final class MainFrame extends ui.MainFrameBase
{
	//
	// クラス属性
	//
	private static final DateFormat firstDateFormat = new SimpleDateFormat("yyyy-MM-dd(EE)");
	private static final DateFormat lastDateFormat = new SimpleDateFormat("- dd(EE)");

	private static MainFrame instance = new MainFrame();
	private static String LABEL_SHOWSETYEARDIALOG;

	//
	// クラスメソッド(公開)
	//

	// クラス初期化子
	static
	{
		// するかどうかもわからない多言語対応を意識してクラス初期化子で初期化する
		LABEL_SHOWSETYEARDIALOG = LabelManager.getLabelProperties().getProperty(LabelManager.LABEL_SELECT);
	}

	// 唯一のインスタンスを返す
	public static MainFrame getInstance()
	{
		return instance;
	}

	//
	// クラスメソッド(パッケージ)
	//

	// 祝日の一覧を作成する
	static Set getHolidaysOfYear(
		HolidayInfoList holidays,	// 祝日定義リスト
		int nYear)					// 対象年
	{
		if (holidays == null) {
			throw new NullPointerException();
		}

		Set dayOfHolidays = new TreeSet();

		List holidaysOfYear = holidays.getHolidaysOfYear(nYear);
		if (holidaysOfYear.size() == 0) {
			return dayOfHolidays;
		}
		
		int nIndex = holidaysOfYear.size() - 1;
		HolidayInfo.DateOfYear holiday = (HolidayInfo.DateOfYear)holidaysOfYear.get(nIndex);
		while (0 < nIndex) {
			if (dayOfHolidays.isEmpty()) {
				dayOfHolidays.addAll(holiday.getAdditionalDays(dayOfHolidays));
			}
			dayOfHolidays.add(holiday);
			nIndex--;
			HolidayInfo.DateOfYear prevHoliday = (HolidayInfo.DateOfYear)holidaysOfYear.get(nIndex);
			dayOfHolidays.addAll(prevHoliday.getAdditionalDays(dayOfHolidays));
			holiday = prevHoliday;
			if (nIndex == 0) {
				dayOfHolidays.add(holiday);
			}
		}
		return dayOfHolidays;
	}

	// 祝日情報を文字列化して返す
	static String formatHoliday(
		HolidayInfo.DateOfYear holiday) // 祝日情報
	{
		StringBuffer buffer = new StringBuffer();
		buffer.append(firstDateFormat.format(holiday.date()));
		if (1 < holiday.days()) {
			Calendar dateOfYear = holiday.calendar();
			dateOfYear.roll(Calendar.DATE, false);
		//	dateOfYear.add(Calendar.DATE, holiday.days() - 1);
			buffer.append(lastDateFormat.format(dateOfYear.getTime()));
		}

		buffer.append(" - ");
		buffer.append(holiday.name());

		return buffer.toString();
	}

	// 祝日文字列の一覧を作成する
	static List getHolidaysText(
		HolidayInfoList holidays,	// 祝日定義リスト
		int nYear)					// 対象年
	{
		Set dayOfHolidays = getHolidaysOfYear(holidays, nYear);

		// 文字列に変換する
		List holidaysText = new ArrayList();
		Iterator it = dayOfHolidays.iterator();
		while (it.hasNext()) {
			holidaysText.add(formatHoliday((HolidayInfo.DateOfYear)it.next()));
		}

		return holidaysText;
	}

	// 引数から年を取得する
	static int getYearFromArgument(
		ArgumentInfo argument)	// 引数
	{
		if (argument == null) {
			throw new NullPointerException();
		}
		
		Calendar now = Calendar.getInstance();
		
		int nYear;
		if (argument.hasOption(SBHoliday.OPTION_YEAR)) {
			String strYear = argument.getOptionValue(SBHoliday.OPTION_YEAR);
			try {
				nYear = Math.max(Integer.parseInt(strYear), 0);
			}
			catch (NumberFormatException e) {
				System.err.print("Caution: Illegal " + SBHoliday.OPTION_YEAR + " option(." + strYear + ").");
				nYear = now.get(Calendar.YEAR);
			}
		}
		else {
			nYear = now.get(Calendar.YEAR);
		}
		return nYear;
	}
	//
	// 属性
	//
	private HolidayInfoList holidays;
	private int currentYear;

	//
	// 構築子
	//
	private MainFrame() {
		super(true);
	}

	//
	// 操作
	//

	// 祝日情報を設定する
	public void setHolidayInfo(
		HolidayInfoList initialHolidays) // 祝日定義情報(入力)
	{
		holidays = initialHolidays;
	}

	//
	// 実装
	//

	// フレームをMainFrameOperationインターフェースとして返す
	private ImplementOperation getOperation()
	{
		return (ImplementOperation) getImplementor();
	}

	// 実装を初期化する
	private void initImplementor(
		Frame frame) // 作成した実装
	{
		final ImplementOperation op = (ImplementOperation)frame;
		op.initComponent();
		op.setYearChoiceListener(new ItemListener() {
			// 年選択処理
			public void itemStateChanged(ItemEvent e) // イベント情報(入力)
			{
				if (e.getStateChange() == ItemEvent.SELECTED) {
					String strSelectedYear = op.getSelectedYearValue();
					if (strSelectedYear.equals(LABEL_SHOWSETYEARDIALOG)) {
						EventQueue.invokeLater(new Runnable() {
							public void run() {
								showSetYearDialog();
							}
						});
					}
					else {
						setHolidaysAtYearText(Integer.parseInt(strSelectedYear));
					}
				}
			}
		});
		op.setSaveListener(new ActionListener() {
			// 年選択処理
			public void actionPerformed(ActionEvent e) // イベント情報(入力)
			{
				saveHoliday();
			}
		});
	}

	// リストに指定年の祝日を設定する
	private void setHolidaysAtYearText(
		int nYear) // 設定年(入力)
	{
		if (nYear != currentYear) {
			currentYear = nYear;
			getOperation().setHolidayList(getHolidaysText(holidays, currentYear));
		}
	}

	// 祝日の一覧を作成する
	private Set getHolidaysOfYear()
	{
		return getHolidaysOfYear(holidays, currentYear);
	}

	// 年指定ダイアログを表示する
	private void showSetYearDialog()
	{
		int nSetYear = getOperation().showSetYearDialog(currentYear);
		getOperation().setYear(0 < nSetYear ? nSetYear : currentYear);
		if (0 < nSetYear && nSetYear != currentYear) {
			setHolidaysAtYearText(nSetYear);
		}
	}

	// 祝日リストを保存する
	private void saveHoliday()
	{
		String strSavePath = getOperation().getSaveFileFullPath(getFrameTitle("FileSaveDialog"),
																EnvInfo.getInstance().getUserHomeDir(),
																"holiday" + Integer.toString(currentYear) + ".txt");
		if (strSavePath != null) {
			try {
				Serializer serializer = HolidayListSerializer.getInstanceForFile(strSavePath, getHolidaysOfYear());
				serializer.serialize(new FileOutputStream(strSavePath));
			}
			catch (Exception e) {
				showExceptionMessage(e);
			}
		}
	}

	//
	// 再定義メソッド(MainFrameBase)
	//

	// デフォルトのサイズを返す
	protected Dimension getDefaultSize() {
		Frame frame = getImplementor();
		if (frame == null) {
			throw new IllegalStateException();
		}

		Dimension screenSize = frame.getToolkit().getScreenSize();
		return new Dimension(screenSize.width / 3, screenSize.height / 2);
	}

	// アプリケーション名を返す
	protected String getAppName(
		boolean bShort) // 短縮名フラグ
	{
		return (bShort ? "sbholiday" : "StompBox/Holiday for Java");
	}

	// 実装を作成する	
	// 注記: J2MEに存在しないクラスのロードを避けるためにリフレクションで作成する
	protected Frame createImplementor(ArgumentInfo argument) // 引数情報
	{
		Frame frame = null;

		try {
			if (isToolkitAwt(argument) == false) {
				initSwing(argument);
			//	frame = new swing.MainFrameImp();
				frame =	(Frame)Class.forName("view.swing.MainFrameImp").newInstance();
			}
			else {
				frame = new view.awt.MainFrameImp();
			//	frame = (Frame)Class.forName("view.awt.MainFrameImp").newInstance();
			}

			initImplementor(frame);
		}
		catch (ClassNotFoundException exp) {
			// Do nothing.
			outputExceptionInfo(exp);
		}
		catch (IllegalAccessException exp) {
			// Do nothing.
			outputExceptionInfo(exp);
		}
		catch (InstantiationException exp) {
			// Do nothing.
			outputExceptionInfo(exp);
		}

		return frame;
	}

	// 初期化する
	protected boolean doInit(
		ArgumentInfo argument) // 引数情報
	{
		setTitle(getAppName(false) + " Ver.3.14");

		// アイコンを設定する
		setIcon();

		// とりあえず前後十年間を選択リストに設定
		List yearList = new ArrayList();

		yearList.add(LABEL_SHOWSETYEARDIALOG);
		int nYear = getYearFromArgument(argument);
		
		for (int nOffset = -10; nOffset < 10; nOffset++) {
			yearList.add(Integer.toString(nYear + nOffset));
		}
		getOperation().setYearList(yearList, nYear);
		setHolidaysAtYearText(nYear);

		return true;
	}

	// 追加状態を復元する
	protected void restoreAdditionalState()
	{
		try {
			getOperation().setAdditionalState(getStateProperties());
		}
		catch (Exception exp) {
			// Do nothing.
			outputExceptionInfo(exp);
		}
	}

	// 追加状態を設定する
	protected void setAdditionalStatePropreties() {
		Map mapAdditionalState = getOperation().getAdditionalState();
		if (mapAdditionalState != null) {
			Set keySet = mapAdditionalState.keySet();
			Iterator it = keySet.iterator();
			while (it.hasNext()) {
				String strKey = (String)it.next();
				setStatePropertyValue(strKey, (String)mapAdditionalState.get(strKey));
			}
		}
	}

	// 終了可能かを返す
	protected boolean canExit() {
		// 常に終了可
		return true;
	}

	// 終了準備処理を行う
	protected void prepareExit() {
		getOperation().prepareExit();
	}
}
